/*
 *  Copyright (C) 1998 Universita` degli Studi di Pisa.
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and agree to license at no charge to all parties under these terms and
 * conditions any derivative works or modified versions of this software.
 * 
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE COPYRIGHT HOLDERS DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL THE COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

package it.unipi.di.utils;

import java.lang.reflect.Modifier;
import java.lang.reflect.Method;

import java.io.LineNumberReader;
import java.io.FileReader;

import java.util.StringTokenizer;
import java.util.Vector;

/**
 * <script>
 * <!--
 *   document.writeln('<i>Last update:</i> '+document.lastModified);
 * //-->
 * </script><p>
 * <b>Note:</b> This class works only with Java 1.1<p>
 * With this class the user can start multiple java applications on a single JVM.
 * If the user starts an application with<p>
 * <ul><i>java class params</i></ul><p>
 * the user can add the line<p>
 * <ul><i>class params</i></ul><p>
 * to the JavaStarter configuration file. The JavaStarter starts a Thread for each
 * line in the configuration file.
 * <p>For the moment this application has some limit:<p><ul>
 * <li> Params with spaces are not allowed, also if they are enclosed between " or '.
 * <li> Settings of java environment with the command line must be specified when the
 *      JavaStarter application starts.
 * </ul><p>
 * JavaStarter is a very simple application that is very useful when one has several
 * services written in Java that must run on the same server. With JavaStarter it is
 * possible to share a single JVM among many applications.
 * <p><h3>Future works</h3>I plan to extend JavaStarter with a simple TCP based interface
 * that allows to start new Java applications when the JVM is already running.
 * @author Antonio Cisternino
 * @version 1.0
 */
public class JavaStarter implements Runnable {
  /**
   * Name of the configuration file. The name of the file is <i><b>application.list</b></i>
   */
  public static String fileName = "application.list";

  /**
   * Reference to the method main. Is used by the thread to invoke the main method of
   * an application.
   */
  private Method m;

  /**
   * List of arguments for the method main (contains only one element)
   */
  private Object[] arg;

  /**
   * Constructor of the starter. Is private because it is invoked only by the
   * main method of theis class.
   * @param m Method main to start
   * @param args Arguments list for the method main
   */
  private JavaStarter(Method m, String[] args) {
    this.m = m;
    arg = new Object[1];
    arg[0] = args;
  }

  /**
   * Used by the JavaStarter to start the various applications.
   */
  public void run() {
    try {
      m.invoke(null, arg);
    } catch (Exception e) {
    }
  }

  /**
   * Print the help on the screen and exit.
   */
  public static void usage() {
    System.out.println("JavaStarter, (C)1998, Universita` di Pisa");
    System.out.println("Usage: java JavaStarter [config file]");
    System.exit(0);
  }

  /**
   * This class parse the configuration file and starts various threads. If something
   * goes wrong the application halts the JVM.
   * @param args Arguments passed by command line
   */
  public static void main(String[] args) {
    // Pars of the command line
    if (args.length > 1)
      usage();

    if ((args.length == 1) && args[0].equals("-h"))
      usage();

    if (args.length == 1)
      fileName = args[0];
    
    // Starts the variuos applications
    String line = "";

    try {
      LineNumberReader in = new LineNumberReader(new FileReader(fileName));

      while ((line = in.readLine()) != null) {
        if (line.equals(""))
          continue;

        Vector v = new Vector();
        StringTokenizer st = new StringTokenizer(line, " ");
        String className = st.nextToken();

        while (st.hasMoreTokens())
          v.addElement(st.nextToken());

        String[] cargs = new String[v.size()];
        for (int i = 0; i < v.size(); i++)
          cargs[i] = (String) v.elementAt(i);

        Class c = Class.forName(className);
        Class[] sign = { Class.forName("java.lang.String") };
        
        Method ma[] = c.getMethods(), m = null;
        for (int i = 0; i < ma.length; i++)
          if (ma[i].getName().equals("main") &&
              Modifier.isStatic(ma[i].getModifiers()) &&
              (ma[i].getParameterTypes().length == 1)) {
            m = ma[i];
            break;
          }

        if (m == null) throw new Exception("Method main not found!");
        
        Thread t = new Thread(new JavaStarter(m, cargs));
        t.start();
      }
    } catch (Exception e) {
      System.out.println("[JavaStarter: error in line "+line+".]");
      System.exit(1);
    }
  }
}

